@@ -17,46 +17,36 @@
 # along with GNOME DVB Daemon.  If not, see <http://www.gnu.org/licenses/>.
 
 import gtk
-import pango
 import gobject
 from gettext import gettext as _
 import gnomedvb
 from gnomedvb.ui.widgets.ChannelsStore import ChannelsStore
 from gnomedvb.ui.widgets.ChannelsView import ChannelsView
+from gnomedvb.ui.widgets.RunningNextStore import RunningNextStore
+from gnomedvb.ui.widgets.RunningNextView import RunningNextView
+from gnomedvb.ui.widgets.SchedulePaned import SchedulePaned
 from gnomedvb.ui.widgets.ScheduleStore import ScheduleStore
-from gnomedvb.ui.widgets.ScheduleView import ScheduleView
+from gnomedvb.ui.widgets.HelpBox import HelpBox
+from gnomedvb.ui.channellisteditor.ChannelListEditorDialog import ChannelListEditorDialog
 from gnomedvb.ui.timers.EditTimersDialog import EditTimersDialog
-from gnomedvb.ui.timers.TimerDialog import NoTimerCreatedDialog
+from gnomedvb.ui.timers.MessageDialogs import TimerFailureDialog
 from gnomedvb.ui.preferences.Preferences import Preferences
 from gnomedvb.ui.recordings.RecordingsDialog import RecordingsDialog
-
-class HelpBox(gtk.EventBox):
-
-    def __init__(self):
-        gtk.EventBox.__init__(self)
-        self.modify_bg(gtk.STATE_NORMAL, self.style.base[gtk.STATE_NORMAL])
-                
-        frame = gtk.Frame()
-        frame.set_shadow_type(gtk.SHADOW_IN)
-        self.add(frame)
-        
-        self._helpview = gtk.Label()
-        self._helpview.set_ellipsize(pango.ELLIPSIZE_END)
-        self._helpview.set_alignment(0.50, 0.50)
-        frame.add(self._helpview)
-        
-    def set_markup(self, helptext):
-        self._helpview.set_markup("<span foreground='grey50'>%s</span>" % helptext)
-
-
+   
 class ControlCenterWindow(gtk.Window):
 
     def __init__(self, model):
         gtk.Window.__init__(self)
         
+        self.runningnextstore = None
+        self.scrolledrunningnext = None
+        self.runningnextview = None
+        self.__single_group = None
+        
         self.channellists = {}
         self.manager = model
-        self.manager.connect('changed', self._on_manager_changed)
+        self.manager.connect('group-added', self._on_manager_group_added)
+        self.manager.connect('group-removed', self._on_manager_group_removed)
         
         self.connect('delete-event', gtk.main_quit)
         self.connect('destroy-event', gtk.main_quit)
@@ -82,11 +72,8 @@ class ControlCenterWindow(gtk.Window):
         
         self.vbox_left = gtk.VBox(spacing=6)
         self.hpaned.pack1(self.vbox_left)
-        
-        groups_label = gtk.Label(_("Device groups:"))
-        self.vbox_left.pack_start(groups_label, False)
-        
-        self.devgroupslist = gtk.ListStore(str, int)
+
+        self.devgroupslist = gtk.ListStore(str, int, gobject.TYPE_PYOBJECT)
         self.devgroupslist.connect("row-inserted", self._on_devgroupslist_inserted)
         
         self.devgroupscombo = gtk.ComboBox(self.devgroupslist)
@@ -95,7 +82,6 @@ class ControlCenterWindow(gtk.Window):
         cell_adapter = gtk.CellRendererText()
         self.devgroupscombo.pack_start(cell_adapter)
         self.devgroupscombo.add_attribute(cell_adapter, "markup", 0)
-        
         self.vbox_left.pack_start(self.devgroupscombo, False)
         
         self.channelsstore = None
@@ -114,24 +100,22 @@ class ControlCenterWindow(gtk.Window):
                 
         self.help_eventbox = HelpBox()
         self.choose_group_text = _("Choose a device group and channel on the left to view the program guide")
-        self.create_group_text = _("No device groups are configured. Please go to preferences and create one.")
+        self.create_group_text = _("No devices are configured. Please go to preferences to configure them.")
+        self.no_events_text = _("There is currently no schedule available for this channel")
         self.hpaned.pack2(self.help_eventbox)
         
-        self.scheduleview = ScheduleView()
+        self.schedulepaned = SchedulePaned()
+        self.schedulepaned.show()
+        self.scheduleview = self.schedulepaned.get_treeview()
         self.scheduleview.connect("button-press-event", self._on_event_selected)
-        self.scheduleview.show()
-        
-        self.scrolledschedule = gtk.ScrolledWindow()
-        self.scrolledschedule.add(self.scheduleview)
-        self.scrolledschedule.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        self.scrolledschedule.set_shadow_type(gtk.SHADOW_IN)
-        self.scrolledschedule.show()
         
         self.get_device_groups()
         if len(self.devgroupslist) == 0:
             self.help_eventbox.set_markup(self.create_group_text)
         else:
             self._select_first_group()
+
+        gtk.window_set_default_icon_name("gnome-dvb-daemon")
       
     def __create_menu(self):
         ui = '''
@@ -143,9 +127,12 @@ class ControlCenterWindow(gtk.Window):
             <menuitem action="Quit"/>
           </menu>
           <menu action="Edit">
+            <menuitem action="EditChannelLists"/>
+            <separator/>
             <menuitem action="Preferences"/>
           </menu>
           <menu action="View">
+            <menuitem action="WhatsOnNow"/>
             <menuitem action="Refresh"/>
             <menuitem action="PrevDay"/>
             <menuitem action="NextDay"/>
@@ -167,7 +154,7 @@ class ControlCenterWindow(gtk.Window):
         # Create actions
         actiongroup = gtk.ActionGroup('Root')
         actiongroup.add_actions([
-            ('Timers', None, _('_Timers')),
+            ('Timers', None, _('_Recording schedule')),
             ('Edit', None, _('_Edit')),
             ('View', None, _('_View')),
             ('Help', None, _('Help')),
@@ -177,9 +164,9 @@ class ControlCenterWindow(gtk.Window):
         
         actiongroup = gtk.ActionGroup('Timers')
         actiongroup.add_actions([
-            ('EditTimers', None, _('_Manage'), None,
-             _('Manage timers'), self._on_button_display_timers_clicked),
-            ('Recordings', None, _('_Recordings'), None,
+            ('EditTimers', None, _('_Manage'), '<Control>T',
+             _('Manage recording schedule'), self._on_button_display_timers_clicked),
+            ('Recordings', None, _('_Recordings'), '<Control>W',
              _('Manage recordings'), self._on_button_recordings_clicked),
             ('Quit', gtk.STOCK_QUIT, _('_Quit'), None,
              _('Quit the Program'), gtk.main_quit)])
@@ -187,6 +174,8 @@ class ControlCenterWindow(gtk.Window):
         
         actiongroup = gtk.ActionGroup('Edit')
         actiongroup.add_actions([
+            ('EditChannelLists', None, _('_Channel Lists'), None,
+             _('Edit channel lists'), self._on_edit_channellists_clicked),
             ('Preferences', gtk.STOCK_PREFERENCES, _('_Preferences'), None,
              _('Display preferences'), self._on_button_prefs_clicked),
         ])
@@ -194,17 +183,19 @@ class ControlCenterWindow(gtk.Window):
         
         actiongroup = gtk.ActionGroup('View')
         actiongroup.add_actions([
-            ('Refresh', gtk.STOCK_REFRESH, _('Refresh'), None,
+            ('WhatsOnNow', None, _("_What's on now"), None,
+             _("See what's currently on and is coming next"), self._on_whats_on_now_clicked),
+            ('Refresh', gtk.STOCK_REFRESH, _('_Refresh'), '<Control>R',
              _('Refresh program guide'), self._on_refresh_clicked),
-            ('PrevDay', None, _('Previous Day'), '<Control>B',
+            ('PrevDay', None, _('_Previous Day'), '<Alt>Left',
              _('Go to previous day'), self._on_button_prev_day_clicked),
-            ('NextDay', None, _('Next Day'), '<Control>N',
+            ('NextDay', None, _('_Next Day'), '<Alt>Right',
              _('Go to next day'), self._on_button_next_day_clicked),
         ])
         actiongroup.add_toggle_actions([
-            ('Channels', None, _('Channels'), None,
+            ('Channels', None, _('_Channels'), None,
              _('View/Hide channels'), self._on_view_channels_clicked),
-            ('Toolbar', None, _('Toolbar'), None,
+            ('Toolbar', None, _('_Toolbar'), None,
              _('View/Hide toolbar'), self._on_view_toolbar_clicked),
         ])
         action = actiongroup.get_action('Toolbar')
@@ -213,7 +204,7 @@ class ControlCenterWindow(gtk.Window):
         action.set_active(True)
         uimanager.insert_action_group(actiongroup, 3)
         
-        actiongroup = gtk.ActionGroup('Edit')
+        actiongroup = gtk.ActionGroup('Help')
         actiongroup.add_actions([
             ('About', gtk.STOCK_ABOUT, _('_About'), None,
              _('Display informations about the program'),
@@ -223,24 +214,26 @@ class ControlCenterWindow(gtk.Window):
 
         # Add a UI description
         uimanager.add_ui_from_string(ui)
-        
-        icon_theme = gtk.icon_theme_get_default()
-        
-        pixbuf = icon_theme.load_icon("stock_timer", gtk.ICON_SIZE_MENU, gtk.ICON_LOOKUP_USE_BUILTIN)
-        timers_image = gtk.image_new_from_pixbuf(pixbuf)
+
+        timers_image = gtk.image_new_from_icon_name("stock_timer", gtk.ICON_SIZE_MENU)
         timers_image.show()
         
         self.timersitem = uimanager.get_widget('/MenuBar/Timers/EditTimers')
         self.timersitem.set_image(timers_image)
         self.timersitem.set_sensitive(False)
         
-        pixbuf = icon_theme.load_icon("video", gtk.ICON_SIZE_MENU, gtk.ICON_LOOKUP_USE_BUILTIN)
-        recordings_image = gtk.image_new_from_pixbuf(pixbuf)
+        recordings_image = gtk.image_new_from_icon_name("video", gtk.ICON_SIZE_MENU)
         recordings_image.show()
         
         recordings = uimanager.get_widget('/MenuBar/Timers/Recordings')
         recordings.set_image(recordings_image)
         
+        whatson_image = gtk.image_new_from_stock(gtk.STOCK_INDEX, gtk.ICON_SIZE_MENU)
+        whatson_image.show()
+        
+        whatons_item = uimanager.get_widget('/MenuBar/View/WhatsOnNow')
+        whatons_item.set_image(whatson_image)
+        
         self.refresh_menuitem = uimanager.get_widget('/MenuBar/View/Refresh')
         self.refresh_menuitem.set_sensitive(False)
         
@@ -265,38 +258,45 @@ class ControlCenterWindow(gtk.Window):
         self.toolbar = gtk.Toolbar()
         self.toolbar.show()
         self.vbox_outer.pack_start(self.toolbar, False)
-        
-        icon_theme = gtk.icon_theme_get_default()
-        
-        pixbuf = icon_theme.load_icon("stock_timer", gtk.ICON_SIZE_LARGE_TOOLBAR, gtk.ICON_LOOKUP_USE_BUILTIN)
-        timers_image = gtk.image_new_from_pixbuf(pixbuf)
+
+        timers_image = gtk.image_new_from_icon_name("stock_timer", gtk.ICON_SIZE_MENU)
         timers_image.show()
         
-        self.button_display_timers = gtk.ToolButton(icon_widget=timers_image, label=_("Timers"))
+        self.button_display_timers = gtk.ToolButton(icon_widget=timers_image, label=_("Recording schedule"))
+        self.button_display_timers.set_is_important(True)
         self.button_display_timers.set_sensitive(False)
         self.button_display_timers.connect("clicked", self._on_button_display_timers_clicked)
-        self.button_display_timers.set_tooltip_markup(_("Manage timers"))
+        self.button_display_timers.set_tooltip_markup(_("Manage recording schedule"))
         self.button_display_timers.show()
         self.toolbar.insert(self.button_display_timers, 0)
         
-        pixbuf = icon_theme.load_icon("video", gtk.ICON_SIZE_MENU, gtk.ICON_LOOKUP_USE_BUILTIN)
-        recordings_image = gtk.image_new_from_pixbuf(pixbuf)
+        recordings_image = gtk.image_new_from_icon_name("video", gtk.ICON_SIZE_MENU)
         recordings_image.show()
         
         button_recordings = gtk.ToolButton(icon_widget=recordings_image, label=_("Recordings"))
+        button_recordings.set_is_important(True)
         button_recordings.connect("clicked", self._on_button_recordings_clicked)
         button_recordings.show()
         self.toolbar.insert(button_recordings, 1)
+        
+        whatson_image = gtk.image_new_from_stock(gtk.STOCK_INDEX, gtk.ICON_SIZE_LARGE_TOOLBAR)
+        whatson_image.show()
+        
+        button_whatson = gtk.ToolButton(icon_widget=whatson_image, label=_("What's on now"))
+        button_whatson.set_is_important(True)
+        button_whatson.connect("clicked", self._on_whats_on_now_clicked)
+        button_whatson.show()
+        self.toolbar.insert(button_whatson, 2)
          
         sep = gtk.SeparatorToolItem()
         sep.show()
-        self.toolbar.insert(sep, 2)
+        self.toolbar.insert(sep, 3)
         
         self.refresh_button = gtk.ToolButton(gtk.STOCK_REFRESH)
         self.refresh_button.connect("clicked", self._on_refresh_clicked)
         self.refresh_button.set_tooltip_markup(_("Refresh program guide"))        
         self.refresh_button.show()
-        self.toolbar.insert(self.refresh_button, 3)
+        self.toolbar.insert(self.refresh_button, 4)
         
         prev_image = gtk.image_new_from_stock(gtk.STOCK_GO_BACK, gtk.ICON_SIZE_LARGE_TOOLBAR)
         prev_image.show()
@@ -305,7 +305,7 @@ class ControlCenterWindow(gtk.Window):
         self.button_prev_day.set_tooltip_markup(_("Go to previous day"))
         self.button_prev_day.set_sensitive(False)
         self.button_prev_day.show()
-        self.toolbar.insert(self.button_prev_day, 4)
+        self.toolbar.insert(self.button_prev_day, 5)
         
         next_image = gtk.image_new_from_stock(gtk.STOCK_GO_FORWARD, gtk.ICON_SIZE_LARGE_TOOLBAR)
         next_image.show()
@@ -314,19 +314,32 @@ class ControlCenterWindow(gtk.Window):
         self.button_next_day.set_tooltip_markup(_("Go to next day"))
         self.button_next_day.set_sensitive(False)
         self.button_next_day.show()
-        self.toolbar.insert(self.button_next_day, 5)
+        self.toolbar.insert(self.button_next_day, 6)
         
     def get_device_groups(self):
-        for group in self.manager.get_registered_device_groups():
-            self._append_group(group)
+        def append_groups(groups):
+            for group in groups:
+                self._append_group(group)
+            self.check_single_group_mode()
+        self.manager.get_registered_device_groups(reply_handler=append_groups)
+        
+    def check_single_group_mode(self):
+        val = len(self.devgroupslist) == 1
+        if val:
+            aiter = self.devgroupslist.get_iter_first()
+            self.__single_group = self.devgroupslist[aiter][2]
+            self.devgroupscombo.hide()
+        else:
+            self.__single_group = None
+            self.devgroupscombo.show()
     
     def _select_first_group(self):
         self.devgroupscombo.set_active(0)
         self.channelsview.grab_focus()
            
     def _append_group(self, group):
-        self.devgroupslist.append([group["name"], group["id"]])
-        self.channellists[group["id"]] = gnomedvb.DVBChannelListClient(group["id"])
+        self.devgroupslist.append([group["name"], group["id"], group])
+        self.channellists[group["id"]] = group.get_channel_list()
         
     def _remove_group(self, group_id):
         aiter = None
@@ -335,7 +348,7 @@ class ControlCenterWindow(gtk.Window):
                 aiter = row.iter
                 
         if aiter != None:
-            if self._get_selected_group_id() == group_id:
+            if self._get_selected_group()["id"] == group_id:
                 # Select no group
                 self.devgroupscombo.set_active(-1)
                 
@@ -345,29 +358,37 @@ class ControlCenterWindow(gtk.Window):
     def _reset_ui(self):
         self.channelsstore = None
         self.channelsview.set_model(None)
-        self._reset_schedule_view()
+        if len(self.devgroupslist) == 0:
+            text = self.create_group_text
+        else:
+            text = self.choose_group_text
+        self._display_help_message(text)
         self._set_timers_sensitive(False)
         
     def _reset_schedule_view(self):
         self.schedulestore = None
         self.scheduleview.set_model(None)
-        self._display_help_message()
+        self._display_running_next()
 
-    def _on_manager_changed(self, manager, group_id, change_type):
-        if change_type == 0:
-            # added
-            group = self.manager.get_device_group(group_id)
+    def _on_manager_group_added(self, manager, group_id):
+        group = self.manager.get_device_group(group_id)
+        if group:
             self._append_group(group)
-        elif change_type == 1:
-            # deleted
-            self._remove_group(group_id)
+            self.check_single_group_mode()
+        
+    def _on_manager_group_removed(self, manager, group_id):
+        self._remove_group(group_id)
+        self.check_single_group_mode()
+        self._select_first_group()
             
-    def _get_selected_group_id(self):
+    def _get_selected_group(self):
+        if self.__single_group != None:
+            return self.__single_group
         aiter = self.devgroupscombo.get_active_iter()
         if aiter == None:
             return None
         else:
-            return self.devgroupslist[aiter][1]
+            return self.devgroupslist[aiter][2]
         
     def _get_selected_channel_sid(self):
         model, aiter = self.channelsview.get_selection().get_selected()
@@ -378,11 +399,11 @@ class ControlCenterWindow(gtk.Window):
             return None
     
     def _on_devgroupscombo_changed(self, combo):
-        group_id = self._get_selected_group_id()
-        if group_id != None:
+        group = self._get_selected_group()
+        if group != None:
             self._set_timers_sensitive(True)
             
-            self.channelsstore = ChannelsStore(group_id)
+            self.channelsstore = ChannelsStore(group)
             self.channelsview.set_model(self.channelsstore)
             
             self._reset_schedule_view()
@@ -399,23 +420,29 @@ class ControlCenterWindow(gtk.Window):
         child = self.hpaned.get_child2()
         if aiter != None:
             sid = model[aiter][model.COL_SID]
-            group_id = self._get_selected_group_id()
-            self.schedulestore = ScheduleStore(self.manager.get_schedule(group_id, sid))
+            group = self._get_selected_group()
+            self.schedulestore = ScheduleStore(group, sid)
+            self.schedulestore.connect("loading-finished",
+                self._on_schedule_loading_finished)
+                
             self.scheduleview.set_model(self.schedulestore)
-            
             # Display schedule if it isn't already displayed
-            if child != self.scrolledschedule:
+            if child != self.schedulepaned:
                 self.hpaned.remove(child)
-                self.hpaned.pack2(self.scrolledschedule)
+                self.hpaned.pack2(self.schedulepaned)
                 self._set_previous_day_sensitive(True)
                 self._set_next_day_sensitive(True)
                 self._set_refresh_sensitive(True)
         else:
-            # Display help message if it isn't already displayed
-            if child != self.help_eventbox:
-                self._display_help_message()
-                
-    def _display_help_message(self):
+            # Display running/next if it isn't already displayed
+            if child != self.scrolledrunningnext:
+                self._display_running_next()
+    
+    def _on_schedule_loading_finished(self, schedulestore):
+        if len(self.schedulestore) == 0:
+            self._display_help_message(self.no_events_text)
+            
+    def _display_help_message(self, text):
         child = self.hpaned.get_child2()
         self.hpaned.remove(child)
         self.hpaned.pack2(self.help_eventbox)
@@ -423,10 +450,28 @@ class ControlCenterWindow(gtk.Window):
         self._set_next_day_sensitive(False)
         self._set_refresh_sensitive(False)
         
-        if len(self.devgroupslist) == 0:
-            self.help_eventbox.set_markup(self.create_group_text)
-        else:
-            self.help_eventbox.set_markup(self.choose_group_text)
+        self.help_eventbox.set_markup(text)
+     
+    def _display_running_next(self):
+        group = self._get_selected_group()
+        
+        self.runningnextstore = RunningNextStore(group)
+        
+        self.scrolledrunningnext = gtk.ScrolledWindow()
+        self.scrolledrunningnext.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        self.scrolledrunningnext.set_shadow_type(gtk.SHADOW_IN)
+        self.scrolledrunningnext.show()
+        
+        self.runningnextview = RunningNextView(self.runningnextstore)
+        self.runningnextview.show()
+        self.scrolledrunningnext.add(self.runningnextview)
+    
+        child = self.hpaned.get_child2()
+        self.hpaned.remove(child)
+        self.hpaned.pack2(self.scrolledrunningnext)
+        self._set_previous_day_sensitive(False)
+        self._set_next_day_sensitive(False)
+        self._set_refresh_sensitive(False)
                 
     def _set_next_day_sensitive(self, val):
         self.button_next_day.set_sensitive(val)
@@ -441,8 +486,8 @@ class ControlCenterWindow(gtk.Window):
         self.timersitem.set_sensitive(val)
         
     def _set_refresh_sensitive(self, val):
-       self.refresh_button.set_sensitive(val) 
-       self.refresh_menuitem.set_sensitive(val)
+        self.refresh_button.set_sensitive(val) 
+        self.refresh_menuitem.set_sensitive(val)
        
     def _on_event_selected(self, treeview, event):
         if event.type == gtk.gdk._2BUTTON_PRESS:
@@ -451,27 +496,33 @@ class ControlCenterWindow(gtk.Window):
                 dialog = gtk.MessageDialog(parent=self,
                     flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
                     type=gtk.MESSAGE_QUESTION, buttons=gtk.BUTTONS_YES_NO)
-                dialog.set_markup (_("<big><span weight=\"bold\">Schedule recording for the selected event?</span></big>"))
+                dialog.set_markup (
+                    "<big><span weight=\"bold\">%s</span></big>" % _("Schedule recording for the selected event?"))
+                rec_id = -1
+                success = True
                 if dialog.run() == gtk.RESPONSE_YES:
                     event_id = model[aiter][model.COL_EVENT_ID]
-                    group_id = self._get_selected_group_id()
+                    group = self._get_selected_group()
                     channel_sid = self._get_selected_channel_sid()
-                    recorder = gnomedvb.DVBRecorderClient(group_id)
-                    rec_id = recorder.add_timer_for_epg_event(event_id, channel_sid)
+                    recorder = group.get_recorder()
+                    rec_id, success = recorder.add_timer_for_epg_event(event_id, channel_sid)
                 dialog.destroy()
                 
-                if rec_id == 0:
-                    dialog = NoTimerCreatedDialog(self)
+                if not success:
+                    dialog = TimerFailureDialog(self)
                     dialog.run()
                     dialog.destroy()
         
     def _on_button_display_timers_clicked(self, button):
-        group_id = self._get_selected_group_id()
-        if group_id != None:
-            edit = EditTimersDialog(group_id, self)
+        group = self._get_selected_group()
+        if group != None:
+            edit = EditTimersDialog(group, self)
             edit.run()
             edit.destroy()
             
+    def _on_whats_on_now_clicked(self, button):
+        self._reset_schedule_view()
+            
     def _on_refresh_clicked(self, button):
         self.schedulestore.reload_all()
    
@@ -509,8 +560,7 @@ class ControlCenterWindow(gtk.Window):
     
     def _on_button_prefs_clicked(self, button):
         prefs = Preferences(self.manager, self)
-        prefs.run()
-        prefs.destroy()
+        prefs.show()
         
     def _on_button_recordings_clicked(self, button):
         dialog = RecordingsDialog(self)
@@ -544,4 +594,8 @@ class ControlCenterWindow(gtk.Window):
         about.run()
         about.destroy()
         
-    
+    def _on_edit_channellists_clicked(self, action):
+        dialog = ChannelListEditorDialog(self.manager, parent=self)
+        dialog.run()
+        dialog.destroy()
+
